"""
Stoyled

Standard Output Styled
"""


__version__ = "0.9"
__author__ = "naryal2580"


from .moreStyles import info, \
                        good, \
                        warn, \
                        bad, \
                        coolInput, \
                        takenInput, \
                        coolExit, \
                        fetchFormatedTime, \
                        bold, \
                        dim, \
                        italic, \
                        uline, \
                        blink, \
                        normal, \
                        invert, \
                        hidden, \
                        strike, \
                        red, \
                        green, \
                        yellow, \
                        blue, \
                        purple, \
                        cyan, \
                        white, \
                        red_l, \
                        green_l, \
                        yellow_l, \
                        blue_l, \
                        purple_l, \
                        cyan_l, \
                        white_l, \
                        rst


from os import name

# Adding ANSI Escape Sequence support to Windows devices, check will be made better after I'll enum all reutrn of `os.name`
if name != 'posix':
    from ctypes import windll
    windll.kernel32.SetConsoleMode(windll.kernel32.GetStdHandle(-11), 7)
    del windll


del generateStyles, moreStyles, escapeSequences, name


def _generateEscapeSequence(styleSequence, preNum=0):
    """
    Generate Escape Sequence

        Parameters:
            styleSequence (list): Sequenced list of escape sequence
            preNum (int): Number, that prepends before sequence count
        
        Returns:
            variables (str): Newline separated list consisting set of
                                variables to be later used, instead
                                of escape sequences
    """
    count = 1
    variables = ''
    if preNum:
        postStyle = ''
        if int(preNum) == 9:  # if preNum is 9, then color from escape sequence is light
            postStyle = '_l'
        for style in styleSequence:
            style += postStyle
            variables += f"{style} = '\\x1b[{preNum}{count}m'\n"
            count += 1
    else:
        for _ in styleSequence:
            variables += f"{_} = '\\x1b[{count}m'\n"
            count += 1
    return variables


_STYLES = (
            'bold',
            'dim',
            'italic',
            'uline',
            'blink',
            'normal',
            'invert',
            'hidden',
            'strike'
        )

_COLORS = (
            'red',
            'green',
            'yellow',
            'blue',
            'purple',
            'cyan',
            'white'
        )


exec(_generateEscapeSequence(_STYLES))
exec(_generateEscapeSequence(_COLORS, 3))
exec(_generateEscapeSequence(_COLORS, 9))


rst = '\x1b[0m'  # Fixed escape sequence :p



from .escapeSequences import *


_FORMATS = {
    'info': [
            '*',
            'cyan',
            'blue_l'
            ],
    'warn': [
            '!',
            'yellow_l',
            'yellow'
            ],
    'good': [
            '+',
            'green_l',
            'green'
            ],
    'bad': [
            '-',
            'red_l',
            'red'
            ],
    'takenInput': [
                    '<',
                    'purple_l',
                    'italic'
                    ]
        }


def _generateFunction(_format, formats):
    """
    Generate Fucntion, based on specified format from dict above (_FORMATS)

        Parameters:
            _format (str): Format for this function, to generate funcion from `formats` (dict) above

        Returns:
            func (str): Returns a string, which is a function to `exec()`
    """
    func = 'def '+_format+'(text,color=True):\n\t"""\n\tRich style '+_format+'\n\n\t\tNote:\n\t\t\tIf you want to print `['+formats[_format][0]+'] foo: bar`, then pass text parameter as `foo -> bar` and see the fanciness\n\n\t\tParameters:\n\t\t\ttext (str): Text to be printed\n\t\t\tcolor (bool): Shall colors be used via ANSI Escape Sequence\n\t\t\n\t\tReturns:\n\t\t\tstr: Fancily styled string, which is just ready to print!\n\t"""\n\ttext=str(text)\n\tif len(text.split(\' -> \'))==2:\n\t\t\ttext=text.split(\' -> \')\n\tif color:\n\t\tif type(text)==str:\n\t\t\treturn f\'{rst}[{rst+bold+'+formats[_format][1]+'}'+formats[_format][0]+'{rst+white}] {rst+'+formats[_format][2]+'}{text}{rst}\'\n\t\telif type(text)==list:\n\t\t\treturn f\'{rst+white}[{rst+bold+'+formats[_format][1]+'}'+formats[_format][0]+'{rst+white}]{rst+'+formats[_format][2]+'} {text[0]}: {rst+'+formats[_format][1]+'+italic}{text[1]}{rst}\'\n\telse:\n\t\tif type(text)==str:\n\t\t\treturn f\'['+formats[_format][0]+'] {text}\'\n\t\telif type(text)==list:\n\t\t\treturn f\'['+formats[_format][0]+'] {text[0]}: {text[1]}\''
    return func


for _format in _FORMATS:
    func = _generateFunction(_format, _FORMATS)
    exec(func)



from .generateStyles import *


def fetchFormatedTime():
    """
    Fetch formatted time, returns current date and time in a formatted manner

        Returns:
            now (str): Formatted date and time.
    """
    from datetime import datetime
    now = datetime.now()
    dd = str(now.day)
    mm = str(now.month)
    yyyy = str(now.year)
    HH = str(now.hour)
    MM = str(now.minute)
    SS = str(now.second)
    now = (dd, mm, yyyy, HH, MM, SS)
    now = now[0] + '.' + now[1] + '.' + now[2] + ' ' + now[3] + ':' + now[4] + ':' + now[5]
    return now


def coolExit(exitCode=0, color=True):
    """
    Just a fancier exit

        Parameters:
            exitCode (int): Exit code
            color (bool): Shall color be printed on halt text
    """
    now = fetchFormatedTime()
    print(info(f'Halted [at] -> {now}', color))
    exit(exitCode)


def coolInput(prompt='Prompt', color=True):
    """
    Fancy input

        Parameters:
            prompt (str): Prompt text
            color: Shall used, while input

        Returns:
            _input (str): Input made by the user
    """
    try:
        if color:
            prompt = f'{ rst + white }[{ bold + purple_l }<{ rst + white }]{ rst + purple_l } { prompt }: { rst + white + italic }'
        else:
            prompt = f'[<] {prompt}: '
        _input = input(prompt)
        if color:
            print(rst, end='')
        return _input
    except KeyboardInterrupt:
        print('\b\b' + bold + italic + 'Null' + rst)
        print(bad('Exitting -> recvd. SIGINT'))
        coolExit(1)
    except EOFError:
        print(italic + 'Null' + rst)
        print(bad('Terminating -> EOFError'))
        coolExit(1)



from setuptools import setup, find_packages

with open("./README.md") as f:
    long_description = f.read()

setup(
    name='stoyled',
    version='0.9',
    author="Captain Nick Lucifer",
    author_email="naryal2580@gmail.com",
    url="https://github.com/naryal2580/stoyled",
    download_url='https://github.com/naryal2580/stoyled/tarball/master',
    description="Simple Fancy Standard Outputs for python3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    py_modules=['stoyled'],
    classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Environment :: Console",
         "Operating System :: OS Independent",
     ],
    python_requires='>=3',
 )
